==> spread-5.6.1.59-py3-none-any.whl: spread_core/projector/base.py <==
import os
import json
import logging
from spread_core import mqtt
from spread_core.bam.engineries.lighting_area import ButtonPreset
from spread_lom.profiles import DimmingProfile


class UnsupportedTopic(Exception):
    pass


class VerifyError(Exception):
    pass


class ChangeStorage:
    """ Abstract storage of project changes """

    @property
    def topics(self):
        """ Generator of storage topics """

        raise NotImplementedError()

    def save(self, topic, data):
        """
        Save topic and data in storage. If data is None, delete topic. """

        raise NotImplementedError()

    def load(self, topic):
        """
        Return data of topic in storage. Return None, if topic not in storage. """

        raise NotImplementedError()


class FileSystemStorage(ChangeStorage):
    """ Class for working with file system storage """

    def __init__(self, root, project_id):
        self.root = root  # root не включает в себя project_id
        self.project_id = project_id
        self.dest = os.path.join(root, str(project_id), mqtt.project.LOCAL)

    @property
    def topics(self):
        for dirpath, dirnames, filenames in os.walk(self.dest):
            dest = dirpath.replace(self.dest, '', 1)
            for file in filenames:
                topic_str = mqtt.project.Local.of_file(self.project_id, dest, file)
                topic_str = topic_str.replace(os.path.sep, '/')
                try:
                    topic = mqtt.topic_of(topic_str)
                except:
                    logging.debug('Storage got wrong entity: "%s"' % topic_str)
                    continue
                yield topic

    def save(self, topic: mqtt.project.Local, data):
        file_path = topic.path(self.root)
        file_dir = os.path.dirname(file_path)
        if data is None:
            if os.path.exists(file_path):
                os.remove(file_path)
                if not os.listdir(file_dir):
                    os.removedirs(file_dir)
        else:
            if not os.path.exists(file_dir):
                os.makedirs(file_dir, exist_ok=True)
            with open(file_path, mode='w') as file:
                try:
                    json.dump(data, file, indent=2)
                except json.JSONDecodeError as ex:
                    raise json.JSONDecodeError(f'JSONEncode error of topic "{topic}". {ex.msg}', ex.doc, ex.pos)

    def load(self, topic: mqtt.project.Local):
        file_path = topic.path(self.root)
        if os.path.exists(file_path):
            with open(file_path, mode='rb') as file:
                try:
                    return json.load(file)
                except json.JSONDecodeError as ex:
                    raise json.JSONDecodeError(f'JSONDecode error of file "{file_path}". {ex.msg}', ex.doc, ex.pos)
        return None


def del_empty_dirs(path):
    for d in os.listdir(path):
        a = os.path.join(path, d)
        if os.path.isdir(a):
            del_empty_dirs(a)
            if not os.listdir(a):
                os.rmdir(a)


async def file_sys_storage_modify(client, root, project_id):
    change_list = {
        'Local/Equipment/Lighting/LightingArea/ButtonPresets':
            mqtt.project.Preset(project_id, ButtonPreset.__name__, 1).path(root).rsplit('/', 1)[0],
        'Local/Hardware/Lom/Profiles':
            mqtt.project.Preset(project_id, DimmingProfile.__name__, 1).path(root).rsplit('/', 1)[0],
        'Local/Incubator/Cycles':
            mqtt.project.Cycle(project_id, 1).path(root).rsplit('/', 1)[0]
    }

    for src, dst_path in change_list.items():
        src_path = os.path.join(root, str(project_id), src)
        if os.path.exists(src_path):
            for file in os.listdir(src_path):
                file_path = os.path.join(src_path, file)
                if os.path.isfile(file_path):
                    if src == 'Local/Incubator/Cycles':
                        with open(file_path, mode='rb') as f:
                            try:
                                cycle_data = json.load(f)
                                if isinstance(cycle_data, dict):
                                    triggers = cycle_data.get('triggers')
                                else:
                                    triggers = None

                                if triggers:
                                    for tr in triggers:
                                        await client.publish(f'Project/{project_id}/{src}/{os.path.splitext(file)[0]}/Triggers/{tr}', payload=bytes(), retain=True)

                            except json.JSONDecodeError as ex:
                                raise json.JSONDecodeError(f'JSONDecode error of file "{file_path}". {ex.msg}', ex.doc, ex.pos)

                    if not os.path.exists(dst_path):
                        os.makedirs(dst_path, exist_ok=True)

                    # В spread_dashboard файлы Cycle создаются без расширения!!!
                    # if not os.path.splitext(file)[1]:
                    #     os.replace(os.path.join(src_path, file), os.path.join(dst_path, f'{file}.json'))
                    # else:
                    os.replace(file_path, os.path.join(dst_path, file))

                    await client.publish(f'Project/{project_id}/{src}/{os.path.splitext(file)[0]}', payload=bytes(), retain=True)

            if not os.listdir(src_path):
                os.removedirs(src_path)

==> spread-5.7.0.72-py3-none-any.whl: spread_core/projector/base.py <==
import os
import json
import logging
from spread_core import mqtt
from spread_core.bam.engineries.lighting_area import ButtonPreset
from spread_lom.profiles import DimmingProfile


class UnsupportedTopic(Exception):
    pass


class VerifyError(Exception):
    pass


class ChangesStorage:
    """ Abstract storage of project changes """

    @property
    def topics(self):
        """ Generator of storage topics """

        raise NotImplementedError()

    def save(self, topic, data):
        """
        Save topic and data in storage. If data is None, delete topic. """

        raise NotImplementedError()

    def load(self, topic):
        """
        Return data of topic in storage. Return None, if topic not in storage. """

        raise NotImplementedError()


class FileSystemStorage(ChangesStorage):
    """ Class for working with file system storage """

    def __init__(self, root, project_id):
        self.root = root  # root не включает в себя project_id
        self.project_id = project_id
        self.dest = os.path.join(root, str(project_id), mqtt.project.LOCAL)

    @property
    def topics(self):
        for dirpath, dirnames, filenames in os.walk(self.dest):
            dest = dirpath.replace(self.dest, '', 1)
            for file in filenames:
                topic_str = mqtt.project.Local.of_file(self.project_id, dest, file)
                topic_str = topic_str.replace(os.path.sep, '/')
                try:
                    topic = mqtt.topic_of(topic_str)
                except:
                    logging.debug('Storage got wrong entity: "%s"' % topic_str)
                    continue
                yield topic

    def save(self, topic: mqtt.project.Local, data):
        file_path = topic.path(self.root)
        file_dir = os.path.dirname(file_path)
        if data is None:
            if os.path.exists(file_path):
                os.remove(file_path)
                if not os.listdir(file_dir):
                    os.removedirs(file_dir)
        else:
            if not os.path.exists(file_dir):
                os.makedirs(file_dir, exist_ok=True)
            with open(file_path, mode='w') as file:
                try:
                    json.dump(data, file, indent=2)
                except json.JSONDecodeError as ex:
                    raise json.JSONDecodeError(f'JSONEncode error of topic "{topic}". {ex.msg}', ex.doc, ex.pos)

    def load(self, topic: mqtt.project.Local):
        file_path = topic.path(self.root)
        if os.path.exists(file_path):
            with open(file_path, mode='rb') as file:
                try:
                    return json.load(file)
                except json.JSONDecodeError as ex:
                    raise json.JSONDecodeError(f'JSONDecode error of file "{file_path}". {ex.msg}', ex.doc, ex.pos)
        return None


def del_empty_dirs(path):
    for d in os.listdir(path):
        a = os.path.join(path, d)
        if os.path.isdir(a):
            del_empty_dirs(a)
            if not os.listdir(a):
                os.rmdir(a)


async def file_sys_storage_modify(client, root, project_id):
    change_list = {
        'Local/Equipment/Lighting/LightingArea/ButtonPresets':
            mqtt.project.Preset(project_id, ButtonPreset.__name__, 1).path(root).rsplit('/', 1)[0],
        'Local/Hardware/Lom/Profiles':
            mqtt.project.Preset(project_id, DimmingProfile.__name__, 1).path(root).rsplit('/', 1)[0],
        'Local/Incubator/Cycles':
            mqtt.project.Cycle(project_id, 1).path(root).rsplit('/', 1)[0]
    }

    for src, dst_path in change_list.items():
        src_path = os.path.join(root, str(project_id), src)
        if os.path.exists(src_path):
            for file in os.listdir(src_path):
                file_path = os.path.join(src_path, file)
                if os.path.isfile(file_path):
                    if src == 'Local/Incubator/Cycles':
                        with open(file_path, mode='rb') as f:
                            try:
                                cycle_data = json.load(f)
                                if isinstance(cycle_data, dict):
                                    triggers = cycle_data.get('triggers')
                                else:
                                    triggers = None

                                if triggers:
                                    for tr in triggers:
                                        await client.publish(f'Project/{project_id}/{src}/{os.path.splitext(file)[0]}/Triggers/{tr}', payload=bytes(), retain=True)

                            except json.JSONDecodeError as ex:
                                raise json.JSONDecodeError(f'JSONDecode error of file "{file_path}". {ex.msg}', ex.doc, ex.pos)

                    if not os.path.exists(dst_path):
                        os.makedirs(dst_path, exist_ok=True)

                    # В spread_dashboard файлы Cycle создаются без расширения!!!
                    # if not os.path.splitext(file)[1]:
                    #     os.replace(os.path.join(src_path, file), os.path.join(dst_path, f'{file}.json'))
                    # else:
                    os.replace(file_path, os.path.join(dst_path, file))

                    await client.publish(f'Project/{project_id}/{src}/{os.path.splitext(file)[0]}', payload=bytes(), retain=True)

            if not os.listdir(src_path):
                os.removedirs(src_path)
